# -*- coding: utf-8 -*-

require 'kconv'
require 'cgi'
require 'html_entity'

module Util_2ch

  HASSINMOTO = "発信元"
  def u(str)
    ret = NKF.nkf("--ic=CP932 --oc=utf-8 -x",str)
    ret.force_encoding("utf-8") if ret.respond_to?(:force_encoding)
    ret
  end
  module_function :u

  class Message

    def initialize(name="",mail="",date="",text="",title="")
      @name  = name
      @mail  = mail

      if date=~/^(.+) ((?:ID|HOST|#{HASSINMOTO}):[^\s]+)( .+)?/uo
        @date  = $1
        @uid   = $2
        @uid2  = $3
      else
        @date  = date
        @uid   = nil
        @uid2  = nil
      end
      @text  = text
      @title = title

      @post_number = nil

      @count_by_id   = 0
      @count_by_trip = 0
      @trip = nil
      @id_is_valid = true
    end
    attr_accessor :name,:mail,:date,:text,:title,:uid,:uid2,:post_number
    attr_accessor :count_by_id ,:trip , :count_by_trip , :id_is_valid
    
    DATE1=%r!(\d{2,4})/(\d\d)/(\d\d).*(\d\d):(\d\d):(\d\d)(?:.(\d\d))?!u
    def date_as_timeobj
      if m = DATE1.match( date )
        #puts "date_as_timeobj: #{m.to_a.inspect}"
        begin
          # エイプリルフールの日付ジョーク回避
          month = m[2].to_i
          day = m[3].to_i
          if month == 3 and day > 31
            month = 4
            day -= 31
          end
          t = Time.local( m[1].to_i , month , day ,
                          m[4].to_i , m[5].to_i , m[6].to_i , m[7].to_i * 10000)
          #puts "to time #{t}"
          t
        rescue ArgumentError
          $stderr.puts "util_2ch.rb 日付解析エラー: #{m[0]}"
          nil
        end
      else
        $stderr.puts "util_2ch.rb 日付がマッチしない #{date.to_s}"
        nil
      end
    end

    TRIP_MARK_UTF8 = "◆"
    def self.parse(text,plaintext=false )
      text=text.chomp
      if text=~/<>/uo
        x=text.chomp.split(/<>/uo,-1) 
      else
        x=text.chomp.split(/,/uo,-1)
      end

      #とり検出
      m = x[0].to_s.match( /#{TRIP_MARK_UTF8}(\S{10})/uo )
      trip = m.to_a[1]

      if plaintext
        x.map!{ |e| self.removetag(e) } 
      end

      if x.size==5 or x.size == 6 or # 一部のdatで、1のタイトルの後に<>がある
          x.size == 7 # onionちゃんねる
        m = Message.new( *(x[0..4]) )
        m.trip = trip
        m
      else
        puts "*** ERROR x.size=#{x.size} ***"
        nil
      end
    end
    
    def self.removetag(str)
      ret = str.split(/<br>/iuo ).map{ |l| l.strip }.join("\n")
      dont_remove = 'b|font'
      ret.gsub!(/<(?!\/?(?:#{dont_remove})[ >])[^>]+>/iou , '')
      ret = Html_entity::unescape(ret) 
      ret
    end

  end # Message

  def readmsg(io , start_number = 1 , id_hash = {} ,
              to_plaintext = false)
    msgs=[]
    i = -1
    
    data = if io.kind_of?(IO)
             io.read
           else
             io.to_s
           end
    
    u(data).each_line{|text|
      i += 1
      
      msg = Message::parse(text , to_plaintext)
      if not msg
        STDERR.puts "util_2ch.rb :parse failed:" + text.toutf8
        STDERR.print "bytes:"
        text.each_byte{|b| STDERR.print( "%2x " % b ) }
        STDERR.puts
        # next
        msg = Message.new( "util_2ch.rb parse failed","failed","failed","failed","failed")
      end
      msg.post_number = i + start_number
      if msg.uid.to_s == '' or msg.uid =~ /^ID:\?\?\?/uo
        # 無視id
        $stderr.puts( "util_2ch.rb 無視id " + msg.uid.to_s )
        msg.id_is_valid = false
        msg.count_by_id = nil
      else
        if id_hash[ msg.uid ]
          id_hash[ msg.uid ] <<  msg.post_number 
        else
          id_hash[ msg.uid ] = [ msg.post_number ]
        end
        # idごとの通算ポスト回数
        msg.count_by_id = id_hash[ msg.uid ].length
      end
      # トリップ
      if msg.trip
        if id_hash[ msg.trip ]
          id_hash[ msg.trip ] << msg.post_number
        else
          id_hash[ msg.trip ] = [ msg.post_number ]
        end
        msg.count_by_trip = id_hash[ msg.trip ].length
      end

      msgs << msg
    }
    return [ msgs, id_hash ]
  end
  module_function :readmsg

  # b,fontタグは取らない
  def readmsg_to_plaintext(io , start_number = 1 , id_hash = {})
    readmsg(io,start_number,id_hash,true)
  end
  module_function :readmsg_to_plaintext

end

