#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

$LOAD_PATH.unshift "./libs"

require 'bitcoin_rpc'
require 'thread_2ch'
require 'util_2ch'
require 'kconv'
require 'bigdecimal'
require 'fileutils'

###############################################
# 決め打ち設定

RPC_URL = "http://user:pass@127.0.0.1:9332/"
PAY_AMOUNT = 0.05

###############################################

class GiveawayLogger
  def initialize( thread_id )
    @id = thread_id
    @file = "giveaway_log/#{@id}.log"
    FileUtils.mkdir_p "giveaway_log"
    if File.exist?(@file)
      @checked = File.read( @file ).split(/\n/).map{|s| s.to_i }
    else
      @checked = []
    end
  end

  def check( resnum )
    @checked << resnum
    @checked = @checked.sort.uniq
    open( @file , 'w'){|f| f.write @checked.join("\n")}
  end
  
  def checked?( resnum )
    @checked.find{|n| n == resnum }
  end
end

BASE58_CHARS = "[123456789abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ]"
MONA_ADDRESS_RXP = /M#{BASE58_CHARS}{26,}/
def extract_address( rpc , mes )
  candidates = []
  if mes.name =~ MONA_ADDRESS_RXP
    candidates << Regexp.last_match[0]
  end

  mes.text.scan( MONA_ADDRESS_RXP ).each{ |a|
    candidates << a
  }

  # p candidates
  candidates.find{|a| 
    ret = rpc.validateaddress( a )
    ret['isvalid'] and not ret['ismine']
  }
end

url = ARGV.shift
if not url
  puts "urlを指定してください"
  exit
end

$thread = Thread_2ch.new( url )

thread_id = File.basename( $thread.local_file )
# p thread_id
$logger = GiveawayLogger.new( thread_id )
$rpc = BitcoinRPC.new( RPC_URL )

$total = BigDecimal("0")
PAY = BigDecimal(PAY_AMOUNT.to_s)

loop do
  success,mes = $thread.sync_cache
  if not success
    puts "スレ読み込み失敗"
    puts mes.body
    exit 0
  end

  # 今のところ毎回レスを全部見てる
  fd = open( $thread.local_file , 'r')
  messages , id2res = Util_2ch.readmsg_to_plaintext( fd )
  fd.close
  
  messages.each{|m|
    # m.post_number     レス番
    # m.name            名前欄
    # m.mail            メール欄
    # m.date            書き込み時間(文字列)
    # m.date_as_timeobj 書き込み時間(Timeオブジェクト or nil)
    # m.uid             ID or nil
    # m.trip            トリップ or nil
    # m.count_by_id     このidでの通算書き込み数 or nil

    if $logger.checked?( m.post_number )
      # 処理済み
    else
      puts "新レス: #{m.post_number}"
      if ad = extract_address( $rpc , m )
        puts "対象アドレス: #{ad}"
        $rpc.sendtoaddress( ad , PAY.to_f )
        $total += PAY
        puts "現在の支払い合計: #{$total.to_f}"
      end
      $logger.check( m.post_number )
    end
  }

  if messages.length > 1000
    puts "1000行った"
    exit
  end

  GC.start
  sleep(20)
end # loop
